// INP decompress tester

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>
#include "zlib/zlib.h"

#define BUFFER_SIZE 1024

// header at the front of INP files
struct inp_header
{
	char                        header[8];      // +00: 8 byte header - must be "MAMEINP\0"
	unsigned long long          basetime;       // +08: base time of recording
	unsigned char               majversion;     // +10: major INP version
	unsigned char               minversion;     // +11: minor INP version
	unsigned char               reserved[2];    // +12: must be zero
	char                        gamename[12];   // +14: game name string, NULL-terminated
	char                        version[32];    // +20: system version string, NULL-terminated
};

int main(int argc, void** argv)
{
	FILE* in;
	FILE* out;
	int ret;
	int amount;
	unsigned char inbuffer[BUFFER_SIZE];
	unsigned char outbuffer[BUFFER_SIZE];
	z_stream stream;

	printf("INPTEST - MAME INP file decompressor (%s)\n\n",__DATE__);
	memset(&stream,0,sizeof(z_stream));

	if(argc != 3)
	{
		printf("Usage: INPTEST <MAME INP file to decompress> <decompressed output file>\n\n");
		return -1;
	}

	printf("Input file: %s\nOutput file: %s\n\n",(char*)argv[1],(char*)argv[2]);

	in = fopen(argv[1],"rb");
	out = fopen(argv[2],"wb");

	if(in == NULL)
	{
		printf("Error: Input file cannot be opened\n\n");
		return -2;
	}

	if(out == NULL)
	{
		printf("Error: Output file cannot be opened\n\n");
		return -3;
	}

	fread(inbuffer,sizeof(struct inp_header),1,in);
	fwrite(inbuffer,sizeof(struct inp_header),1,out);

	ret = inflateInit(&stream);
	if(ret != Z_OK)
	{
		printf("Error: Could not initialise zlib stream.\n\n");
		return -4;
	}
	
	// while stream or input file hasn't ended
	do
	{
		int x;
		stream.avail_in = fread(inbuffer,1,BUFFER_SIZE,in);
		if(stream.avail_in == 0)
			break;
		stream.next_in = inbuffer;
		do
		{
			stream.avail_out = BUFFER_SIZE;
			stream.next_out = outbuffer;
			// decompress some data
			ret = inflate(&stream,Z_SYNC_FLUSH);
			// check for errors in decompression
			assert(ret != Z_STREAM_ERROR);
			switch(ret)
			{
				case Z_DATA_ERROR:
					printf("Error: Decompression data error.\n\n");
					inflateEnd(&stream);
					fclose(in);
					fclose(out);
					return -10;
				case Z_MEM_ERROR:
					printf("Error: Decompression memory error.\n\n");
					inflateEnd(&stream);
					fclose(in);
					fclose(out);
					return -11;
				case Z_NEED_DICT:
					printf("Error: Decompression needs dictionary (this shouldn't happen...)\n\n");
					inflateEnd(&stream);
					fclose(in);
					fclose(out);
					return -12;
			}
			// write to output file
			amount = BUFFER_SIZE - stream.avail_out;
			fwrite(outbuffer,1,amount,out);
		} while(stream.avail_out == 0);

	} while(ret != Z_STREAM_END);

	inflateEnd(&stream);

	fclose(in);
	fclose(out);
	printf("Done.\n\n");
}
